# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.heancms import HeanCMS


class Modescanlator(HeanCMS):
    id = 'modescanlator'
    name = 'Mode Scanlator'
    lang = 'pt_BR'
    status = 'disabled'

    base_url = 'https://site.modescanlator.net'
    api_url = 'https://api.modescanlator.net'

    cover_css_path = '#content div.container:first-child > div > div:last-child img'
    authors_css_path = 'div.flex:-soup-contains("Author") > span:last-child'
    synopsis_css_path = 'div.text-muted-foreground > div:nth-child(1)'



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.manga_stream import MangaStream


class Mangakoleji(MangaStream):
    id = 'mangakoleji'
    name = 'Manga Koleji'
    lang = 'tr'
    status = 'disabled'

    base_url = 'https://mangakoleji.com'
    logo_url = base_url + '/wp-content/uploads/2024/08/cropped-manga-koleji-com-32x32.png'

    authors_selector = '.tsinfo .imptdt:-soup-contains("Çizer") i, .tsinfo .imptdt:-soup-contains("Yazar") i'
    genres_selector = '.info-desc .mgen a'
    scanlators_selector = '.tsinfo .imptdt:-soup-contains("Yayınlayan") i'
    status_selector = '.tsinfo .imptdt:-soup-contains("Durum") i'
    synopsis_selector = '[itemprop="description"]'



# SPDX-FileCopyrightText: 2021-2024 Lili Kurek
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Lili Kurek <lilikurek@proton.me>

from komikku.servers.multi.guya import Guya


class Magicaltranslators(Guya):
    id = 'magicaltranslators'
    name = 'Magical Translators'
    lang = 'en'

    base_url = 'https://mahoushoujobu.com'
    logo_url = base_url + '/static/logo-mt-squared-small.png?v=c52226f'
    manga_url = base_url + '/read/manga/{0}/'
    api_manga_url = base_url + '/api/series/{0}/'
    page_image_url = base_url + '/media/manga/{0}/chapters/{1}/{2}'


class Magicaltranslators_pl(Magicaltranslators):
    id = 'magicaltranslators_pl'
    lang = 'pl'



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import json
import logging
from urllib.parse import urlparse

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type

logger = logging.getLogger('komikku.servers.legacyscans')


class Legacyscans(Server):
    id = 'legacyscans'
    name = 'LegacyScans'
    lang = 'fr'
    status = 'disabled'

    base_url = 'https://legacy-scans.com'
    logo_url = base_url + '/imgs/icon.webp'
    api_base_url = 'https://api.legacy-scans.com'
    api_search_url = api_base_url + '/misc/home/search'
    api_latest_updates_url = api_base_url + '/misc/comic/home/updates'
    api_most_popular_url = api_base_url + '/misc/views/monthly'
    manga_url = base_url + '/comics/{0}'
    chapter_url = base_url + '/comics/{0}/{1}'

    api_headers = {
        'Host': urlparse(api_base_url).netloc,
        'Accept': 'image/avif,image/webp,*/*',
        'Referer': f'{base_url}/'
    }

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers = {
                'User-Agent': USER_AGENT,
            }

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],  # not available
            genres=[],
            status=None,  # not available
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        data['name'] = soup.select_one('.serieTitle h1').text.strip()
        data['cover'] = soup.select_one('.serieImg img').get('src')

        #  Details
        for element in soup.select('.serieGenre > span'):
            data['genres'].append(element.text.strip())

        if author1 := soup.select_one('div.serieAdd > p:nth-child(2) > strong'):
            if author1.text.strip() != 'Inconnu':
                data['authors'].append(author1.text.strip())
        if author2 := soup.select_one('div.serieAdd > p:nth-child(3) > strong'):
            if author2.text.strip() != 'Inconnu':
                data['authors'].append(author2.text.strip())

        # Synopsis
        if synopsis_element := soup.select_one('.serieDescription > div'):
            data['synopsis'] = synopsis_element.text.strip()

        # Chapters
        for a_element in reversed(soup.select('.chapterList a')):
            data['chapters'].append(dict(
                slug='/'.join(a_element.get('href').split('/')[-2:]),
                title=a_element.select_one('div span:first-child').text.strip(),
                date=convert_date_string(a_element.select_one('div span:last-child').text.strip(), format='%d/%m/%Y'),
            ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(
            self.chapter_url.format(manga_slug, chapter_slug),
            headers={
                'Referer': self.manga_url.format(manga_slug),
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = dict(
            pages=[],
        )
        for script_element in soup.find_all('script'):
            script = script_element.string
            if not script or 'Reactive' not in script:
                continue

            for datum in json.loads(script):
                if not isinstance(datum, str) or not datum.startswith(f'public/uploads/{manga_slug}'):
                    continue

                data['pages'].append(dict(
                    slug=None,
                    image=f'{self.base_url}/{datum}',
                ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            page['image'],
            headers=self.api_headers,
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_latest_updates(self):
        r = self.session_get(
            self.api_latest_updates_url,
            params=dict(
                start=1,
                end=24,
            ),
            headers=self.api_headers
        )
        if r.status_code != 200:
            return None

        results = []
        for item in r.json()['results']:
            results.append(dict(
                cover='{0}/{1}'.format(self.api_base_url, item['cover']),
                slug=item['slug'],
                name=item['title'],
                last_chapter=item['chapters'][0]['chapterNumber'],
            ))

        return results

    def get_most_populars(self):
        r = self.session_get(
            self.api_most_popular_url,
            headers=self.api_headers
        )
        if r.status_code != 200:
            return None

        results = []
        for item in r.json()['results']:
            results.append(dict(
                cover='{0}/{1}'.format(self.api_base_url, item['cover']),
                slug=item['slug'],
                name=item['title'],
            ))

        return results

    def search(self, term=None, orderby=None):
        r = self.session_get(
            self.api_search_url,
            params=dict(
                title=term,
            ),
            headers=self.api_headers
        )
        if r.status_code != 200:
            return None

        results = []
        for item in r.json()['results']:
            results.append(dict(
                slug=item['slug'],
                name=item['title'],
            ))

        return results



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import json

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT_MOBILE
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type
from komikku.utils import log_error_traceback

#
# MangaSee and MangaLife share exact same content
# MangaSee is the main site, MangaLife is a clone instance used to test new code
#

headers = {
    'User-Agent': USER_AGENT_MOBILE,
}


class Mangasee(Server):
    id = 'mangasee'
    name = 'MangaSee'
    lang = 'en'
    is_nsfw = True
    status = 'disabled'  # 2025/01 replaced by Weeb Central

    base_url = 'https://mangasee123.com'
    search_url = base_url + '/search/'
    manga_url = base_url + '/manga/{0}'
    chapter_url = base_url + '/read-online/{0}-chapter-{1}-page-1.html'
    cover_url = 'https://temp.compsci88.com/cover/{0}.jpg'

    mangas = None

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers = headers

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],  # not available
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=self.cover_url.format(data['slug']),
        ))

        soup = BeautifulSoup(r.text, 'lxml')

        data['name'] = soup.find('h1').text.strip()

        for li_element in soup.find('ul', class_='list-group list-group-flush').find_all('li'):
            if li_element.span is None:
                continue

            label = li_element.span.text.strip()
            li_element.span.extract()

            if label.startswith('Author'):
                data['authors'] = [artist.strip() for artist in li_element.text.split(',')]

            elif label.startswith('Genre'):
                data['genres'] = [genre.strip() for genre in li_element.text.split(',')]

            elif label.startswith('Status'):
                for status in li_element.text.split(','):
                    if 'Scan' not in status:
                        continue

                    status = status.replace('(Scan)', '').strip().lower()
                    if status in ('complete', 'hiatus', 'ongoing', ):
                        data['status'] = status
                    elif status in ('cancelled', 'discontinued', ):
                        data['status'] = 'suspended'
                    break

            elif label.startswith('Description'):
                data['synopsis'] = li_element.text.strip()

        # Chapters
        chapters = None
        try:
            for script in soup.find_all('script'):
                script = script.string
                if not script:
                    continue

                script = script.strip()
                if not script.startswith('function MainFunction'):
                    continue

                for line in script.split('\n'):
                    line = line.strip()
                    if not line.startswith('vm.Chapters'):
                        continue

                    chapters = json.loads(line.split('=')[1].strip()[:-1])
                    break
                break
        except Exception as e:
            log_error_traceback(e)
            return None

        if chapters is not None:
            for chapter in reversed(chapters):
                slug = chapter['Chapter']
                num = int(slug[1:-1])

                title = f'{chapter["Type"]} {num}'
                if chapter['Chapter'][-1] != '0':
                    title = f'{title}.{chapter["Chapter"][-1]}'
                if chapter.get('ChapterName'):
                    title = f'{title} - {chapter["ChapterName"]}'

                data['chapters'].append(dict(
                    slug=slug,
                    title=title,
                    num=num,
                    date=convert_date_string(chapter['Date'], '%Y-%m-%d %H:%M:%S') if chapter.get('Date') else None,
                ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data

        Currently, only pages are expected.
        """
        slug = int(chapter_slug[1:-1])
        if chapter_slug[-1] != '0':
            slug = f'{slug}.{chapter_slug[-1]}'
        if chapter_slug[0] != '1':
            slug = f'{slug}-index-{chapter_slug[0]}'

        r = self.session_get(self.chapter_url.format(manga_slug, slug))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        chapter = None
        domain = None
        try:
            for script in soup.find_all('script'):
                script = script.string
                if not script:
                    continue

                script = script.strip()
                if not script.startswith('jQuery(document).ready(function(n){window.history'):
                    continue

                for line in script.split('\n'):
                    line = line.strip()
                    if not line.startswith('vm.CurChapter') and not line.startswith('vm.CurPathName'):
                        continue

                    if line.startswith('vm.CurChapter'):
                        chapter = json.loads(line.split('=')[1].strip()[:-1])
                    elif line.startswith('vm.CurPathName'):
                        domain = line.split('=')[1].strip()[1:-2]

                    if chapter is not None and domain is not None:
                        break
                break
        except Exception as e:
            log_error_traceback(e)
            return None

        if chapter is None or domain is None:
            return None

        image_prefix = chapter_slug[1:-1]
        if chapter_slug[-1] != '0':
            image_prefix = f'{image_prefix}.{chapter_slug[-1]}'
        if chapter['Directory']:
            image_prefix = f'{chapter["Directory"]}/{image_prefix}'

        data = dict(
            pages=[],
        )
        for index in range(int(chapter['Page'])):
            data['pages'].append(dict(
                slug=None,
                image='https://{0}/manga/{1}/{2}-{3:03d}.png'.format(domain, manga_slug, image_prefix, index + 1),
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(page['image'], headers={'Referer': self.base_url})
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_latest_updates(self):
        results = []

        if self.mangas is None:
            if self.get_most_populars() is None:
                return None

        self.mangas = sorted(self.mangas, key=lambda m: m['lt'], reverse=True)

        for manga in self.mangas[:100]:
            results.append(dict(
                name=manga['s'],
                slug=manga['i'],
                cover=self.cover_url.format(manga['i']),
            ))

        return results

    def get_most_populars(self):
        """
        Returns most popular manga list
        """
        if self.mangas is None:
            r = self.session_get(self.search_url)
            if r.status_code != 200:
                return None

            mime_type = get_buffer_mime_type(r.content)
            if mime_type != 'text/html':
                return None

            soup = BeautifulSoup(r.text, 'lxml')

            try:
                for script in soup.find_all('script'):
                    script = script.string
                    if not script:
                        continue

                    script = script.strip()
                    if not script.startswith('function MainFunction'):
                        continue

                    for line in script.split('\n'):
                        line = line.strip()
                        if not line.startswith('vm.Directory'):
                            continue

                        self.mangas = json.loads(line.split('vm.Directory = ')[1].strip()[:-1])
                        break
                    break
            except Exception as e:
                log_error_traceback(e)
                return None

        if self.mangas is None:
            return None

        self.mangas = sorted(self.mangas, key=lambda m: m['vm'], reverse=True)

        results = []
        for manga in self.mangas[:100]:
            results.append(dict(
                name=manga['s'],
                slug=manga['i'],
                cover=self.cover_url.format(manga['i']),
            ))

        return results

    def search(self, term):
        results = []
        term = term.lower()

        if self.mangas is None:
            if self.get_most_populars() is None:
                return None

        for manga in self.mangas:
            if term not in manga['s'].lower():
                continue

            results.append(dict(
                name=manga['s'],
                slug=manga['i'],
                cover=self.cover_url.format(manga['i']),
            ))

        return results


class Mangalife(Mangasee):
    id = 'mangalife:mangasee'
    name = 'MangaLife'
    lang = 'en'
    status = 'disabled'  # 2025/01 replaced by Weeb Central

    # Clone of MangaSee (with different IPV4 and IPV6 addresses)

    base_url = 'https://manga4life.com'
    search_url = base_url + '/search/'
    manga_url = base_url + '/manga/{0}'
    chapter_url = base_url + '/read-online/{0}-chapter-{1}-page-1.html'



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gettext import gettext as _

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.utils import get_buffer_mime_type
from komikku.utils import is_number


class Mangapill(Server):
    id = 'mangapill'
    name = 'Mangapill'
    lang = 'en'
    is_nsfw = True

    base_url = 'https://mangapill.com'
    logo_url = base_url + '/static/favicon/favicon-32x32.png'
    latest_updates_url = base_url + '/chapters'
    most_populars_url = base_url
    search_url = base_url + '/search'
    manga_url = base_url + '/manga/{0}'
    chapter_url = base_url + '/chapters/{0}'

    filters = [
        {
            'key': 'type',
            'type': 'select',
            'name': _('Type'),
            'description': _('Filter by Type'),
            'value_type': 'single',
            'default': 'all',
            'options': [
                {'key': 'all', 'name': _('All')},
                {'key': 'manga', 'name': _('Manga')},
                {'key': 'novel', 'name': _('Novel')},
                {'key': 'one-shot', 'name': _('Oneshot')},
                {'key': 'doujinshi', 'name': _('Doujinshi')},
                {'key': 'manhwa', 'name': _('Manhwa')},
                {'key': 'manhua', 'name': _('Manhua')},
                {'key': 'oel', 'name': _('OEL')},
            ],
        },
    ]

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers = {
                'User-Agent': USER_AGENT,
            }

    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content

        Initial data should contain at least manga's url (provided by search)
        """
        assert 'url' in initial_data, 'url is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['url']))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],  # not available
            scanlators=[],  # not available
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        info_element = soup.select_one('div.container:nth-child(3) > .flex-col')

        data['name'] = info_element.find('h1').text.strip()
        data['cover'] = info_element.find('img', class_='object-cover').get('data-src')

        #  Details
        for element in info_element.select('.grid > div'):
            label = element.label.text.strip()
            if label == 'Status':
                status = element.div.text
                if status == 'publishing':
                    data['status'] = 'ongoing'
                elif status == 'finished':
                    data['status'] = 'complete'
                elif status == 'on hiatus':
                    data['status'] = 'hiatus'
                elif status == 'discontinued':
                    data['status'] = 'suspended'

        for element in info_element.select('.mb-3:nth-child(4) a'):
            data['genres'].append(element.text.strip())

        # Synopsis
        if synopsis_element := info_element.find('p'):
            data['synopsis'] = synopsis_element.text.strip()

        # Chapters
        for a_element in reversed(soup.select('#chapters a')):
            num = a_element.get('href').split('/')[-1].split('-')[-1]

            data['chapters'].append(dict(
                url='/'.join(a_element.get('href').split('/')[-2:]),
                slug=a_element.get('href').split('/')[-1],
                title=a_element.text.strip(),
                num=num if is_number(num) else None,
                date=None,  # not available
            ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(self.chapter_url.format(chapter_url))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = dict(
            pages=[],
        )
        for img_element in soup.find_all('img', class_='js-page'):
            data['pages'].append(dict(
                slug=None,
                image=img_element.get('data-src'),
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            page['image'],
            headers={
                'Referer': self.manga_url.format(manga_slug)
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(url)

    def get_latest_updates(self, type=None):
        """
        Returns latest updates
        """
        r = self.session_get(self.latest_updates_url)
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select('.container .grid > div'):
            a_element = element.select_one('a.text-secondary')
            results.append(dict(
                name=a_element.select_one('div:first-child').text.strip(),
                url='/'.join(a_element.get('href').split('/')[-2:]),
                slug=a_element.get('href').split('/')[-1],  # not used
                cover=element.a.figure.img.get('data-src'),
            ))

        return results

    def get_most_populars(self, type=None):
        """
        Returns Trending mangas
        """
        r = self.session_get(self.most_populars_url)
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select('.container:nth-child(5) > .grid > div'):
            a_element = element.select_one('a.relative.block')
            results.append(dict(
                name=element.select_one('.justify-end > a.mb-2 > div').text.strip(),
                url='/'.join(a_element.get('href').split('/')[-2:]),
                slug=a_element.get('href').split('/')[-1],  # not used
                cover=element.a.figure.img.get('data-src'),
            ))

        return results

    def search(self, term, type=None):
        r = self.session_get(
            self.search_url,
            params=dict(
                q=term,
                type=type if type != 'all' else None,
                status=None,
            )
        )

        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select('div.container:nth-child(3) > .grid > div'):
            a_element = element.select_one('a.mb-2')
            results.append(dict(
                name=a_element.select_one('div:first-child').text.strip(),
                url='/'.join(a_element.get('href').split('/')[-2:]),
                slug=a_element.get('href').split('/')[-1],  # not used
                cover=element.a.figure.img.get('data-src'),
            ))

        return results



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import logging

from komikku.servers.multi.madtheme import Madtheme

logger = logging.getLogger(__name__)


class Mangabuddy(Madtheme):
    id = 'mangabuddy'
    name = 'MangaBuddy'
    lang = 'en'

    base_url = 'https://mangabuddy.com'
    logo_url = base_url + '/static/sites/mangabuddy/icons/favicon-32x32.png'
    api_base_url = 'https://mangabuddy.com/api'



# SPDX-FileCopyrightText: 2020-2024 JaskaranSM
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: JaskaranSM

from gettext import gettext as _

import requests

from komikku.consts import USER_AGENT
from komikku.models import Settings
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type

SERVER_NAME = 'ReadManhwa'


class ReadmanhwaException(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class Readmanhwa(Server):
    id = 'readmanhwa'
    name = SERVER_NAME
    lang = 'en'
    long_strip_genres = ['Webtoons', ]
    status = 'disabled'  # Switch to Hentai only https://manhwahentai.to (Madara) 2023-09

    base_url = 'https://www.readmanhwa.com'
    manga_url = base_url + '/' + lang + '/webtoon/{0}'

    api_base_url = base_url + '/api/'
    api_search_url = api_base_url + 'comics?q={0}&per_page=20&nsfw={1}'
    api_most_populars_url = api_base_url + 'comics?page=1&q=&sort=popularity&order=desc&duration=year&nsfw={0}'
    api_manga_url = api_base_url + 'comics/{0}?lang=en&nsfw=true'
    api_manga_chapters_url = api_base_url + 'comics/{0}/chapters?lang=en&nsfw=true'
    api_manga_chapter_images_url = api_base_url + 'comics/{0}/{1}/images?lang=en&nsfw=true'

    filters = [
        {
            'key': 'nsfw',
            'type': 'checkbox',
            'name': _('NSFW Content'),
            'description': _('Whether to show manga containing NSFW content'),
            'default': False,
        },
    ]

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers.update({'user-agent': USER_AGENT})

        # Update NSFW filter default value according to current settings
        if Settings.instance:
            self.filters[0]['default'] = Settings.get_default().nsfw_content

    def do_api_request(self, url):
        resp = self.session.get(url, headers={'X-Requested-With': 'XMLHttpRequest'})
        if get_buffer_mime_type(resp.content) != 'text/plain':
            raise ReadmanhwaException(resp.text)

        return resp.json()

    def get_manga_data(self, initial_data):
        """
        Returns manga data using API requests

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'
        try:
            manga = self.do_api_request(self.api_manga_url.format(initial_data['slug']))
        except (ReadmanhwaException, Exception):
            return None

        if not manga.get('slug', False):
            return None

        try:
            chapters = self.do_api_request(self.api_manga_chapters_url.format(manga['slug']))
        except (ReadmanhwaException, Exception):
            return None

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        # Name & cover
        data['name'] = manga['title']
        data['cover'] = manga['thumb_url']

        # Authors & Artists
        for author in manga['authors']:
            data['authors'].append(author['name'])
        for artist in manga['artists']:
            if artist['name'] not in data['authors']:
                data['authors'].append(artist['name'])

        # Status
        if manga['status'] == 'canceled':
            data['status'] = 'suspended'
        elif manga['status'] == 'onhold':
            data['status'] = 'hiatus'
        else:
            data['status'] = manga['status']

        # Genres
        for genre in manga.get('tags', []):
            data['genres'].append(genre['name'])

        data['synopsis'] = manga['description']

        # Chapters
        for chapter in reversed(chapters):
            data['chapters'].append(dict(
                slug=chapter['slug'],
                title=chapter['name'],
                date=convert_date_string(chapter['added_at'], '%Y-%m-%d'),
            ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data
        """
        try:
            images = self.do_api_request(self.api_manga_chapter_images_url.format(manga_slug, chapter_slug))
        except (ReadmanhwaException, Exception):
            return None

        data = dict(
            pages=[],
        )
        for image in images['images']:
            data['pages'].append(dict(
                slug=None,  # not necessary, we know image url already
                image=image['source_url'],
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(page['image'])
        if r.status_code != 200:
            return None

        buffer = r.content
        mime_type = get_buffer_mime_type(buffer)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=buffer,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    def get_most_populars(self, nsfw):
        """
        Returns Popular Manga
        """
        try:
            resp = self.do_api_request(self.api_most_populars_url.format('true' if nsfw else 'false'))
        except (ReadmanhwaException, Exception):
            return None

        results = []
        for item in resp['data']:
            results.append(dict(
                name=item['title'],
                slug=item['slug'],
            ))

        return results

    def search(self, term, nsfw):
        """
        Returns Manga by search
        """
        try:
            resp = self.do_api_request(self.api_search_url.format(term, 'true' if nsfw else 'false'))
        except (ReadmanhwaException, Exception):
            return None

        results = []
        for manga in resp['data']:
            results.append(dict(
                name=manga['title'],
                slug=manga['slug'],
            ))

        return results



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.madara import Madara2


class Toongod(Madara2):
    id = 'toongod'
    name = 'ToonGod'
    lang = 'en'
    is_nsfw_only = True

    has_cf = True

    date_format = '%d %b %Y'
    series_name = 'webtoon'

    base_url = 'https://www.toongod.org'
    chapter_url = base_url + '/webtoon/{0}/{1}/'
    bypass_cf_url = base_url + '/webtoons/'

    images_src_attr = 'data-src'



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.madara import Madara2


class Mangaweebs(Madara2):
    id = 'mangaweebs'
    name = 'MangaWeebs'
    lang = 'en'
    is_nsfw = True
    status = 'disabled'

    date_format = None  # broken, year is missing!

    # mirror: https://ns2.mangaweebs.in

    base_url = 'https://mangaweebs.in'
    chapters_url = base_url + '/manga/{0}/ajax/chapters/'



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gettext import gettext as _
from gettext import ngettext
import threading

from gi.repository import Gio
from gi.repository import GLib
from gi.repository import GObject

from komikku.utils import log_error_traceback
from komikku.models import create_db_connection
from komikku.models import Manga
from komikku.models import Settings
from komikku.utils import if_network_available


class Updater(GObject.GObject):
    """
    Mangas updater
    """
    __gsignals__ = {
        'manga-updated': (GObject.SignalFlags.RUN_FIRST, None, (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT, bool)),
    }

    current_id = None
    queue = []
    running = False
    servers_down = []
    stop_flag = False
    update_at_startup_done = False
    update_library_flag = False

    def __init__(self, window):
        GObject.GObject.__init__(self)

        self.window = window

    def add(self, mangas):
        if not isinstance(mangas, list):
            mangas = [mangas, ]

        batch = len(mangas) > 1  # is it a batch or a single update?

        for manga in mangas:
            if manga.id not in self.queue and manga.id != self.current_id and manga.server.status == 'enabled':
                self.queue.append((manga.id, batch))

    @if_network_available(only_notify=True)
    def start(self):
        def show_notification(id, title, body=None):
            if Settings.get_default().desktop_notifications:
                notification = Gio.Notification.new(title)
                notification.set_priority(Gio.NotificationPriority.HIGH)
                if body:
                    notification.set_body(body)
                self.window.application.send_notification(id, notification)
            else:
                # Use in-app notification
                self.window.add_notification(f'{title}\n{body}' if body else title)

        def run():
            db_conn = create_db_connection()

            total_chapters = 0
            total_errors = 0
            total_successes = 0

            while self.queue:
                if self.stop_flag is True:
                    break

                manga_id, in_batch = self.queue.pop(0)
                manga = Manga.get(manga_id)

                if manga is None or (not manga.is_local and not self.window.network_available):
                    # Skip manga if not found or if network is not available (except for local which don't require an Internet connection)
                    continue

                if manga.server_id in self.servers_down:
                    continue

                if not manga.server.is_up():
                    self.servers_down.append(manga.server_id)
                    self.window.add_notification(
                        _('{0} seems down. Please try updating later').format(manga.server.name)
                    )
                    continue

                self.current_id = manga_id
                try:
                    success, chapters_changes, synced = manga.update_full(db_conn=db_conn)
                    if success:
                        total_successes += 1
                        if nb_recents := len(chapters_changes['recent_ids']):
                            total_chapters += nb_recents
                        GLib.idle_add(complete, manga, in_batch, chapters_changes, synced)
                    else:
                        total_errors += 1
                        GLib.idle_add(error, manga)
                except Exception as e:
                    user_error_message = log_error_traceback(e)
                    total_errors += 1
                    GLib.idle_add(error, manga, user_error_message)

            self.current_id = None
            self.running = False
            self.servers_down = []

            if not self.update_library_flag and total_successes + total_errors <= 1:
                # If only one or no comic has been updated, it's not necessary to send end notification
                return

            # End notification
            if self.update_library_flag:
                self.update_library_flag = False
                if total_errors > 0:
                    title = _('Library update completed with errors')
                else:
                    title = _('Library update completed')
            else:
                if total_errors > 0:
                    title = _('Update completed with errors')
                else:
                    title = _('Update completed')

            messages = []
            if total_chapters > 0:
                messages.append(ngettext('{0} successful update', '{0} successful updates', total_successes).format(total_successes))
                messages.append(ngettext('{0} new chapter', '{0} new chapters', total_chapters).format(total_chapters))
            else:
                messages.append(_('No new chapters'))

            if total_errors > 0:
                messages.append(ngettext('{0} update failed', '{0} updates failed', total_errors).format(total_errors))

            show_notification('updater.0', title, '\n'.join(messages))

        def complete(manga, in_batch, chapters_changes, synced):
            nb_recent_chapters = len(chapters_changes['recent_ids'])

            if nb_recent_chapters > 0:
                show_notification(
                    f'updater.{manga.id}',
                    manga.name,
                    ngettext('{0} new chapter', '{0} new chapters', nb_recent_chapters).format(nb_recent_chapters)
                )

                # Auto download new chapters
                if Settings.get_default().new_chapters_auto_download and not manga.is_local:
                    self.window.downloader.add(chapters_changes['recent_ids'], emit_signal=True)
                    self.window.downloader.start()
            elif not in_batch:
                show_notification(f'updater.{manga.id}', manga.name, _('No new chapters'))

            self.emit(
                'manga-updated',
                manga,
                dict(
                    nb_recent_chapters=nb_recent_chapters,
                    nb_updated_chapters=chapters_changes['nb_updated'],
                    nb_deleted_chapters=chapters_changes['nb_deleted'],
                ),
                synced
            )

            return False

        def error(manga, message=None):
            show_notification(
                f'updater.{manga.id}',
                manga.name,
                message or _('Oops, update has failed. Please try again.')
            )

            return False

        if self.running or len(self.queue) == 0:
            return False

        if self.update_library_flag:
            title = _('Library update started')
            show_notification('updater.0', title)

        self.running = True
        self.stop_flag = False

        thread = threading.Thread(target=run)
        thread.daemon = True
        thread.start()

        return True

    def remove(self, manga):
        if manga.id in self.queue:
            self.queue.remove(manga.id)

    def stop(self):
        if self.running:
            self.stop_flag = True

    def update_library(self, startup=False):
        if startup:
            self.update_at_startup_done = True

        db_conn = create_db_connection()
        if self.window.network_available:
            # Update all manga in library
            rows = db_conn.execute('SELECT * FROM mangas WHERE in_library = 1 ORDER BY last_read DESC').fetchall()
        else:
            # Offline, update local manga only
            rows = db_conn.execute('SELECT * FROM mangas WHERE in_library = 1 AND server_id = "local" ORDER BY last_read DESC').fetchall()

        if rows:
            self.update_library_flag = True
            self.add([Manga.get(row['id']) for row in rows])
            return self.start()

        return False



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from gi.repository import GLib
from gi.repository import Gtk


class Controls:
    active = False
    is_visible = False
    pages_count = 0
    reader = None

    def __init__(self, reader):
        self.reader = reader
        self.window = reader.window

        self.bottom_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, visible=False)
        self.bottom_box.props.margin_end = 12
        self.bottom_box.props.margin_bottom = 12
        self.bottom_box.props.margin_start = 12
        self.bottom_box.add_css_class('toolbar')
        self.bottom_box.add_css_class('osd')
        self.bottom_box.set_valign(Gtk.Align.END)

        # Number of pages
        self.label = Gtk.Label()
        self.label.add_css_class('numeric')
        self.label.props.margin_start = 6
        self.label.props.margin_end = 6
        self.label.set_halign(Gtk.Align.START)
        self.bottom_box.append(self.label)

        # Chapter's pages slider: current / nb
        self.scale = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, 1, 2, 1)
        self.scale.set_hexpand(True)
        self.scale.set_increments(1, 0)  # Disable scrolling with mouse wheel
        self.scale_handler_id = self.scale.connect('change-value', self.on_scale_value_changed)
        self.scale_timeout_id = None

        self.bottom_box.append(self.scale)
        self.reader.overlay.add_overlay(self.bottom_box)

    def hide(self):
        if self.window.is_fullscreen():
            self.reader.headerbar_revealer.set_reveal_child(False)

        self.is_visible = False
        self.bottom_box.set_visible(False)

    def init(self, chapter):
        self.active = chapter.pages is not None
        if not self.active:
            return

        self.pages_count = len(chapter.pages)

        # Set slider range
        with self.scale.handler_block(self.scale_handler_id):
            self.scale.set_range(1, self.pages_count)

    def on_fullscreen(self):
        self.reader.headerbar_revealer.set_reveal_child(self.is_visible)

    def on_scale_value_changed(self, _scale, scroll_type, value):
        if self.scale_timeout_id:
            GLib.source_remove(self.scale_timeout_id)
            self.scale_timeout_id = None

        def goto_page(index):
            self.reader.pager.goto_page(index)
            self.scale_timeout_id = None

        value = round(value)
        if scroll_type == Gtk.ScrollType.JUMP and value > 0:
            # Schedule event
            self.scale_timeout_id = GLib.timeout_add(250, goto_page, value - 1)

    def on_unfullscreen(self):
        self.reader.headerbar_revealer.set_reveal_child(True)

    def set_scale_value(self, index):
        if not self.active:
            return

        with self.scale.handler_block(self.scale_handler_id):
            self.scale.set_value(index)
            self.label.set_text(f'{index}/{self.pages_count}')

    def set_scale_direction(self, inverted):
        self.scale.set_inverted(inverted)
        self.scale.set_value_pos(Gtk.PositionType.RIGHT if inverted else Gtk.PositionType.LEFT)
        if inverted:
            self.bottom_box.reorder_child_after(self.scale, self.label)
        else:
            self.bottom_box.reorder_child_after(self.label, self.scale)

    def show(self):
        if self.window.is_fullscreen():
            self.reader.headerbar_revealer.set_reveal_child(True)

        if not self.active:
            return

        self.is_visible = True
        self.bottom_box.set_visible(True)



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.madara import Madara2


class Manhuaus(Madara2):
    id = 'manhuaus'
    name = 'Manhuaus'
    lang = 'en'
    is_nsfw = True

    has_cf = True

    base_url = 'https://manhuaus.com'
    chapters_url = base_url + '/manga/{0}/ajax/chapters/'

    images_src_attr = 'data-src'



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

import base64
import datetime
from gettext import gettext as _
import time
import urllib.parse

from bs4 import BeautifulSoup

from komikku.consts import DOWNLOAD_MAX_DELAY
from komikku.servers import Server
from komikku.servers.utils import get_soup_element_inner_text
from komikku.servers.utils import parse_nextjs_hydration
from komikku.utils import get_buffer_mime_type
from komikku.utils import get_response_elapsed
from komikku.utils import is_number
from komikku.webview import CompleteChallenge

SEARCH_RESULTS_PAGES = 4

KEYS = {
    0: '13YDu67uDgFczo3DnuTIURqas4lfMEPADY6Jaeqky+w=',   # 0  RC4 key  round 1
    1: 'yEy7wBfBc+gsYPiQL/4Dfd0pIBZFzMwrtlRQGwMXy3Q=',   # 1  mut_key  round 1
    2: 'yrP+EVA1Dw==',                                   # 2  pref_key round 1
    3: 'vZ23RT7pbSlxwiygkHd1dhToIku8SNHPC6V36L4cnwM=',   # 3  RC4 key  round 2
    4: 'QX0sLahOByWLcWGnv6l98vQudWqdRI3DOXBdit9bxCE=',   # 4  mut_key  round 2
    5: 'WJwgqCmf',                                       # 5  pref_key round 2
    6: 'BkWI8feqSlDZKMq6awfzWlUypl88nz65KVRmpH0RWIc=',   # 6  RC4 key  round 3
    7: 'v7EIpiQQjd2BGuJzMbBA0qPWDSS+wTJRQ7uGzZ6rJKs=',   # 7  mut_key  round 3
    8: '1SUReYlCRA==',                                   # 8  pref_key round 3
    9: 'RougjiFHkSKs20DZ6BWXiWwQUGZXtseZIyQWKz5eG34=',   # 9  RC4 key  round 4
    10: 'LL97cwoDoG5cw8QmhI+KSWzfW+8VehIh+inTxnVJ2ps=',  # 10 mut_key  round 4
    11: '52iDqjzlqe8=',                                  # 11 pref_key round 4
    12: 'U9LRYFL2zXU4TtALIYDj+lCATRk/EJtH7/y7qYYNlh8=',  # 12 RC4 key  round 5
    13: 'e/GtffFDTvnw7LBRixAD+iGixjqTq9kIZ1m0Hj+s6fY=',  # 13 mut_key  round 5
    14: 'xb2XwHNB',                                      # 14 pref_key round 5
}


def generate_hash(path, body_size=0, time=1):
    """
    :param path: API path, e.g. '/manga/some-hash/chapters'
    :param body_size: encodeURIComponent(body) length for POST, or 0 for GET
    :param time: 1 for GET manga requests, `System.currentTimeMillis()` for POST
    """

    def get_key_bytes(index):
        b64 = KEYS.get(index)
        if not b64:
            return []

        key = []
        for c in base64.b64decode(b64):
            key.append(int(c) & 0xFF)

        return key

    def get_mut_key(mk, idx):
        if mk and (idx % 32) < len(mk):
            return mk[idx % 32]
        return 0

    def mut_s(e):
        return (e + 143) % 256

    def mut_l(e):
        return ((e >> 1) | (e << 7)) & 255

    def mut_c(e):
        return (e + 115) % 256

    def mut_m(e):
        return e ^ 177

    def mut_f(e):
        return (e - 188 + 256) % 256

    def mut_g(e):
        return ((e << 2) | (e >> 6)) & 255

    def mut_h(e):
        return (e - 42 + 256) % 256

    def mut_dollar(e):
        return ((e << 4) | (e >> 4)) & 255

    def mut_b(e):
        return (e - 12 + 256) % 256

    def mut_underscore(e):
        return (e - 20 + 256) % 256

    def mut_y(e):
        return ((e >> 1) | (e << 7)) & 255

    def mut_k(e):
        return (e - 241 + 256) % 256

    def RC4(key, data):
        if not key:
            return data
        s = list(range(256))
        j = 0
        for i in range(256):
            j = (j + s[i] + key[i % len(key)]) % 256
            temp = s[i]
            s[i] = s[j]
            s[j] = temp

        i = 0
        j = 0
        out = [0] * len(data)
        for k in range(len(data)):
            i = (i + 1) % 256
            j = (j + s[i]) % 256
            temp = s[i]
            s[i] = s[j]
            s[j] = temp
            out[k] = data[k] ^ s[(s[i] + s[j]) % 256]

        return out

    def round1(data):
        enc = RC4(get_key_bytes(0), data)
        mut_key = get_key_bytes(1)
        pref_key = get_key_bytes(2)
        out = []

        for i, c in enumerate(enc):
            if i < 7 and i < len(pref_key):
                out.append(pref_key[i])

            v = c ^ get_mut_key(mut_key, i)
            if i % 10 in (0, 9):
                v = mut_c(v)
            elif i % 10 == 1:
                v = mut_b(v)
            elif i % 10 == 2:
                v = mut_y(v)
            elif i % 10 == 3:
                v = mut_dollar(v)
            elif i % 10 in (4, 6):
                v = mut_h(v)
            elif i % 10 == 5:
                v = mut_s(v)
            elif i % 10 == 7:
                v = mut_k(v)
            elif i % 10 == 8:
                v = mut_l(v)

            out.append(v & 255)

        return out

    def round2(data):
        enc = RC4(get_key_bytes(3), data)
        mut_key = get_key_bytes(4)
        pref_key = get_key_bytes(5)
        out = []

        for i, c in enumerate(enc):
            if i < 6 and i < len(pref_key):
                out.append(pref_key[i])

            v = c ^ get_mut_key(mut_key, i)
            if i % 10 in (0, 8):
                v = mut_c(v)
            elif i % 10 == 1:
                v = mut_b(v)
            elif i % 10 in (2, 6):
                v = mut_dollar(v)
            elif i % 10 == 3:
                v = mut_h(v)
            elif i % 10 in (4, 9):
                v = mut_s(v)
            elif i % 10 == 5:
                v = mut_k(v)
            elif i % 10 == 7:
                v = mut_underscore(v)

            out.append(v & 255)

        return out

    def round3(data):
        enc = RC4(get_key_bytes(6), data)
        mut_key = get_key_bytes(7)
        pref_key = get_key_bytes(8)
        out = []

        for i, c in enumerate(enc):
            if i < 7 and i < len(pref_key):
                out.append(pref_key[i])

            v = c ^ get_mut_key(mut_key, i)
            if i % 10 == 0:
                v = mut_c(v)
            elif i % 10 == 1:
                v = mut_f(v)
            elif i % 10 in (2, 8):
                v = mut_s(v)
            elif i % 10 == 3:
                v = mut_g(v)
            elif i % 10 == 4:
                v = mut_y(v)
            elif i % 10 == 5:
                v = mut_m(v)
            elif i % 10 == 6:
                v = mut_dollar(v)
            elif i % 10 == 7:
                v = mut_k(v)
            elif i % 10 == 9:
                v = mut_b(v)

            out.append(v & 255)

        return out

    def round4(data):
        enc = RC4(get_key_bytes(9), data)
        mut_key = get_key_bytes(10)
        pref_key = get_key_bytes(11)
        out = []

        for i, c in enumerate(enc):
            if i < 8 and i < len(pref_key):
                out.append(pref_key[i])

            v = c ^ get_mut_key(mut_key, i)
            if i % 10 == 0:
                v = mut_b(v)
            elif i % 10 in (1, 9):
                v = mut_m(v)
            elif i % 10 in (2, 7):
                v = mut_l(v)
            elif i % 10 in (3, 5):
                v = mut_s(v)
            elif i % 10 in (4, 6):
                v = mut_underscore(v)
            elif i % 10 == 8:
                v = mut_y(v)

            out.append(v & 255)

        return out

    def round5(data):
        enc = RC4(get_key_bytes(12), data)
        mut_key = get_key_bytes(13)
        pref_key = get_key_bytes(14)
        out = []

        for i, c in enumerate(enc):
            if i < 6 and i < len(pref_key):
                out.append(pref_key[i])

            v = c ^ get_mut_key(mut_key, i)
            if i % 10 == 0:
                v = mut_underscore(v)
            elif i % 10 in (1, 7):
                v = mut_s(v)
            elif i % 10 == 2:
                v = mut_c(v)
            elif i % 10 in (3, 5):
                v = mut_m(v)
            elif i % 10 == 4:
                v = mut_b(v)
            elif i % 10 == 6:
                v = mut_f(v)
            elif i % 10 == 8:
                v = mut_dollar(v)
            elif i % 10 == 9:
                v = mut_g(v)

            out.append(v & 255)

        return out

    base_str = f'{path}:{body_size}:{time}'  # noqa: E231
    encoded = urllib.parse.quote_plus(base_str).replace('+', '%20').replace('*', '%2A').replace('%7E', '~')

    initial_bytes = []
    for i in encoded.encode():
        initial_bytes.append(int(i) & 0xFF)

    r1 = round1(initial_bytes)
    r2 = round2(r1)
    r3 = round3(r2)
    r4 = round4(r3)
    r5 = round5(r4)

    final_bytes = []
    for c in r5:
        final_bytes.append(c.to_bytes(1))

    return base64.b64encode(b''.join(final_bytes), altchars=b'-_').replace(b'=', b'')


class Comix(Server):
    id = 'comix'
    name = 'Comix'
    lang = 'en'
    is_nsfw = True

    has_cf = True

    base_url = 'https://comix.to'
    logo_url = base_url + '/icon.png?icon.530a1f27.png'
    api_url = 'https://comix.to/api/v2'
    api_search_url = api_url + '/manga'
    manga_url = base_url + '/title/{0}'
    api_chapters_url = api_url + '/manga/{0}/chapters'
    chapter_url = base_url + '/title/{0}/{1}'

    filters = [
        {
            'key': 'statuses',
            'type': 'select',
            'name': _('Status'),
            'description': _('Filter by Statuses'),
            'value_type': 'multiple',
            'options': [
                {'key': 'releasing', 'name': _('Ongoing'), 'default': False},
                {'key': 'finished', 'name': _('Completed'), 'default': False},
                {'key': 'on_hiatus', 'name': _('Hiatus'), 'default': False},
                {'key': 'discontinued', 'name': _('Canceled'), 'default': False},
            ]
        },
        {
            'key': 'types',
            'type': 'select',
            'name': _('Type'),
            'description': _('Filter by Types'),
            'value_type': 'multiple',
            'options': [
                {'key': 'manga', 'name': _('Manga'), 'default': False},
                {'key': 'manhwa', 'name': _('Manhwa'), 'default': False},
                {'key': 'manhua', 'name': _('Manhua'), 'default': False},
                {'key': 'other', 'name': _('Other'), 'default': False},
            ],
        },
        {
            'key': 'demographics',
            'type': 'select',
            'name': _('Publication Demographic'),
            'description': _('Filter by Publication Demographics'),
            'value_type': 'multiple',
            'options': [
                {'key': 3, 'name': _('Josei'), 'default': False},
                {'key': 4, 'name': _('Seinen'), 'default': False},
                {'key': 1, 'name': _('Shoujo'), 'default': False},
                {'key': 2, 'name': _('Shounen'), 'default': False},
            ]
        },
    ]

    long_strip_genres = ['Manhua', 'Manhwa']

    params = [
        {
            'key': 'hide_nsfw',
            'type': 'checkbox',
            'name': _('Hide NSFW Content'),
            'description': _('Hide NSFW content from popular, latest, and search lists'),
            'default': True,
        },
    ]

    def __init__(self):
        self.session = None

    @CompleteChallenge()
    def get_manga_data(self, initial_data):
        """
        Returns manga data by scraping manga HTML page content and API for chapters

        Initial data should contain at least manga's slug (provided by search)
        """
        assert 'slug' in initial_data, 'Manga slug is missing in initial data'

        r = self.session_get(self.manga_url.format(initial_data['slug']))

        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],  # not available
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
            cover=None,
        ))

        data['name'] = soup.select_one('h1.title').text.strip()
        data['cover'] = soup.select_one('img[itemprop="image"]').get('src')

        # Details
        if element := soup.select_one('.status'):
            status = element.text.strip()
            if 'FINISHED' in status:
                data['status'] = 'complete'
            elif 'RELEASING' in status:
                data['status'] = 'ongoing'
            elif 'ON HIATUS' in status:
                data['status'] = 'hiatus'
            elif 'DISCONTINUED' in status:
                data['status'] = 'suspended'

        for element in soup.select('#metadata li:-soup-contains("Authors") a'):
            data['authors'].append(element.text.strip())

        for element in soup.select('#metadata li:-soup-contains("Artists") a'):
            artist = element.text.strip()
            if artist not in data['authors']:
                data['authors'].append(artist)

        for element in soup.select('#metadata li:-soup-contains("Genres") a'):
            data['genres'].append(element.text.strip())

        for element in soup.select('#metadata li:-soup-contains("Type") a'):
            data['genres'].append(element.text.strip())

        if element := soup.select_one('.content'):
            data['synopsis'] = get_soup_element_inner_text(element, sep='\n\n')

        # Chapters
        data['chapters'] = self.get_manga_chapters_data(data['slug'])

        return data

    @CompleteChallenge()
    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns manga chapter data by scraping chapter HTML page content

        Pages URLs are available in a <script> element
        """
        r = self.session_get(self.chapter_url.format(manga_slug, chapter_slug))
        if r.status_code != 200:
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        info = parse_nextjs_hydration(soup, 'images')
        if info is None:
            return None

        data = dict(
            pages=[],
        )
        for image in info[3]['chapter']['images']:
            data['pages'].append(dict(
                slug=None,
                image=image['url'],
            ))

        return data

    def get_manga_chapters_data(self, slug):
        """
        Returns manga chapters data using API
        """
        chapters = []
        hash_id = slug.split('-')[0]
        path = f'/manga/{hash_id}/chapters'
        hash_token = generate_hash(path, 0, 1)

        def get_page(page):
            r = self.session_get(
                self.api_chapters_url.format(hash_id),
                params={
                    'order[number]': 'desc',
                    'limit': 100,
                    'time': 1,
                    'page': page,
                    '_': hash_token,
                },
                headers={
                    'Content-Type': 'application/json',
                    'Referer': self.manga_url.format(slug),
                }
            )
            if r.status_code != 200:
                return None, False, 0

            resp_data = r.json()
            if resp_data['status'] != 200:
                return None, False, 0

            more = page < resp_data['result']['pagination']['last_page']

            return r.json()['result']['items'], more, get_response_elapsed(r)

        chapters = []
        delay = None
        more = True
        page = 1
        while more:
            if delay:
                time.sleep(delay)

            items, more, rtime = get_page(page)
            if not items:
                return []
            for item in items:
                title = f'Ch. {item["number"]}'
                if item['volume']:
                    title = f'{title} Vol {item["volume"]}'
                if item['name']:
                    title = f'{title} {item["name"]}'
                if item['scanlation_group']:
                    scanlators = [item['scanlation_group']['name']]
                elif item['is_official']:
                    scanlators = ['Official']
                else:
                    scanlators = []

                chapters.append(dict(
                    slug=f'{item["chapter_id"]}-chapter-{item["number"]}',
                    title=title,
                    scanlators=scanlators,
                    num=item['number'] if is_number(item['number']) else None,
                    num_volume=item['volume'] if is_number(item['volume']) else None,
                    date=datetime.datetime.fromtimestamp(item['updated_at']).date(),
                ))

            delay = min(rtime * 4, DOWNLOAD_MAX_DELAY) if rtime else None
            page += 1

        return list(reversed(chapters))

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            page['image'],
            headers={
                'Referer': self.chapter_url.format(manga_slug, chapter_slug),
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=page['image'].split('/')[-1],
        )

    def get_manga_list(self, term=None, statuses=None, types=None, demographics=None, orderby=None):
        def get_page(page):
            params = {
                'limit': 28,
                'page': page,
            }

            if term:
                params.update({
                    'keyword': term,
                    'order[relevance]': 'desc',
                })
            elif orderby == 'latest':
                params.update({
                    'order[chapter_updated_at]': 'desc',
                })
            elif orderby == 'popular':
                params.update({
                    'order[views_30d]': 'desc',
                })

            if statuses:
                params['statuses[]'] = statuses
            if types:
                params['types[]'] = types
            if demographics:
                params['demographics[]'] = demographics

            if self.get_param('hide_nsfw'):
                params['genres[]'] = [-87264, -87266, -87268, -87265]

            r = self.session_get(
                self.api_search_url,
                params=params,
                headers={
                    'Referer': f'{self.base_url}/browser',
                }
            )
            if r.status_code != 200:
                return [], False, None

            resp_data = r.json()
            if resp_data['status'] != 200:
                return [], False, None

            more = page < resp_data['result']['pagination']['last_page'] and page < SEARCH_RESULTS_PAGES

            return resp_data['result']['items'], more, get_response_elapsed(r)

        results = []
        delay = None
        more = True
        page = 1
        while more:
            if delay:
                time.sleep(delay)

            items, more, rtime = get_page(page)
            for item in items:
                results.append(dict(
                    slug=f'{item["hash_id"]}-{item["slug"]}',
                    name=item['title'],
                    cover=item['poster']['medium'],
                    last_chapter=item['latest_chapter'],
                    nb_chapters=item['final_chapter'],
                ))

            delay = min(rtime * 4, DOWNLOAD_MAX_DELAY) if rtime else None
            page += 1

        return results

    def get_manga_url(self, slug, url):
        """
        Returns manga absolute URL
        """
        return self.manga_url.format(slug)

    @CompleteChallenge()
    def get_latest_updates(self, statuses=None, types=None, demographics=None):
        """
        Returns latest updated mangas
        """
        return self.get_manga_list(statuses=statuses, types=types, demographics=demographics, orderby='latest')

    @CompleteChallenge()
    def get_most_populars(self, statuses=None, types=None, demographics=None):
        """
        Returns most popular mangas
        """
        return self.get_manga_list(statuses=statuses, types=types, demographics=demographics, orderby='popular')

    @CompleteChallenge()
    def search(self, term, statuses=None, types=None, demographics=None):
        return self.get_manga_list(term=term, statuses=statuses, types=types, demographics=demographics)



# SPDX-FileCopyrightText: 2019-2026 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from komikku.servers.multi.iken import Iken


class Ezmanga(Iken):
    id = 'ezmanga'
    name = 'EZmanga'
    lang = 'en'

    base_url = 'https://ezmanga.org'
    logo_url = base_url + '/ezmanga.ico'
    api_url = 'https://vapi.ezmanga.org/api/v1'



# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
# SPDX-License-Identifier: GPL-3.0-or-later
# Author: Valéry Febvre <vfebvre@easter-eggs.com>

from bs4 import BeautifulSoup
import requests

from komikku.consts import USER_AGENT
from komikku.servers import Server
from komikku.servers.utils import convert_date_string
from komikku.utils import get_buffer_mime_type
from komikku.utils import is_number


class Viewcomics(Server):
    id = 'viewcomics'
    name = 'View Comics'
    lang = 'en'
    status = 'disabled'

    base_url = 'https://azcomix.me'
    search_url = base_url + '/search'
    api_search_url = base_url + '/ajax/search'
    latest_updates_url = base_url + '/comic-updates'
    most_populars_url = base_url + '/popular-comics'
    manga_url = base_url + '/comic/{0}'
    chapter_url = base_url + '/{0}/{1}/full'

    csrf_token = None
    headers_images = {}  # Do not set referer!
    is_nsfw = True

    def __init__(self):
        if self.session is None:
            self.session = requests.Session()
            self.session.headers.update({'user-agent': USER_AGENT})

    def get_manga_data(self, initial_data):
        """
        Returns comic data by scraping comic HTML page content

        Initial data should contain at least comic slug (provided by search)
        """
        assert 'slug' in initial_data, 'Comic slug is missing in initial data'

        r = self.session_get(
            self.manga_url.format(initial_data['slug']),
            headers={
                'referer': self.base_url,
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = initial_data.copy()
        data.update(dict(
            authors=[],
            scanlators=[],  # not available
            genres=[],
            status=None,
            synopsis=None,
            chapters=[],
            server_id=self.id,
        ))

        details_element = soup.find('div', class_='anime-details')

        data['name'] = details_element.find('h1', class_='title').text.strip()
        data['cover'] = details_element.find('div', class_='anime-image').img.get('src')

        # Details
        for li_element in details_element.find('ul', class_='anime-genres').find_all('li'):
            genre = li_element.text.strip()
            if genre in ('Completed', 'Ongoing'):
                if genre == 'Completed':
                    data['status'] = 'complete'
                else:
                    data['status'] = 'ongoing'
            else:
                data['genres'].append(genre)

        data['authors'] = [details_element.find('div', class_='anime-desc').find_all('tr')[3].find_all('td')[1].text.strip(), ]
        data['synopsis'] = soup.find('div', class_='detail-desc-content').p.text.strip()

        # Chapters
        for li_element in reversed(soup.find('ul', class_='basic-list').find_all('li')):
            a_element = li_element.a

            slug = a_element.get('href').split('/')[-1]
            num = slug.split('-')[-1] if slug.startswith('issue-') else None

            data['chapters'].append(dict(
                slug=slug,
                title=a_element.text.strip(),
                num=num if is_number(num) else None,
                date=convert_date_string(li_element.span.text.strip(), '%m/%d/%Y'),
            ))

        return data

    def get_manga_chapter_data(self, manga_slug, manga_name, chapter_slug, chapter_url):
        """
        Returns comic chapter data by scraping chapter HTML page content

        Currently, only pages are expected.
        """
        r = self.session_get(self.chapter_url.format(manga_slug, chapter_slug))
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        data = dict(
            pages=[],
        )

        for index, img_element in enumerate(soup.select('.chapter-container img')):
            data['pages'].append(dict(
                slug=None,
                image=img_element.get('src').strip(),
                index=index + 1,
            ))

        return data

    def get_manga_chapter_page_image(self, manga_slug, manga_name, chapter_slug, page):
        """
        Returns chapter page scan (image) content
        """
        r = self.session_get(
            page['image'],
            headers=self.headers_images,
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if not mime_type.startswith('image'):
            return None

        if page.get('index'):
            name = '{0:03d}.{1}'.format(page['index'], mime_type.split('/')[1])
        else:
            # Fallback, old version
            name = '{0}.{1}'.format(page['image'].split('/')[-1], mime_type.split('/')[1])

        return dict(
            buffer=r.content,
            mime_type=mime_type,
            name=name,
        )

    def get_manga_url(self, slug, url):
        """
        Returns comic absolute URL
        """
        return self.manga_url.format(slug)

    def get_latest_updates(self):
        """
        Returns latest updates
        """
        r = self.session_get(
            self.latest_updates_url,
            headers={
                'referer': self.base_url,
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for a_element in soup.select('.line-list > li > a'):
            results.append(dict(
                slug=a_element.get('href').split('/')[-1],
                name=a_element.text.strip(),
            ))

        return results

    def get_most_populars(self):
        """
        Returns list of most popular comics
        """
        r = self.session_get(
            self.most_populars_url,
            headers={
                'referer': self.base_url,
            }
        )
        if r.status_code != 200:
            return None

        mime_type = get_buffer_mime_type(r.content)
        if mime_type != 'text/html':
            return None

        soup = BeautifulSoup(r.text, 'lxml')

        results = []
        for element in soup.select('.eg-list .eg-box'):
            a_element = element.select_one('.egb-serie')
            img_element = element.select_one('.eg-image img')
            results.append(dict(
                slug=a_element.get('href').split('/')[-1],
                name=a_element.text.strip(),
                cover=img_element.get('src'),
            ))

        return results

    def search(self, term):
        if self.csrf_token is None:
            r = self.session_get(self.search_url)
            if r.status_code != 200:
                return None

            soup = BeautifulSoup(r.text, 'lxml')

            self.csrf_token = soup.select_one('meta[name="csrf-token"]')['content']

        r = self.session_get(
            self.api_search_url,
            params=dict(q=term),
            headers={
                'Referer': self.base_url,
                'X-Csrf-TOKEN': self.csrf_token,
                'X-Requested-With': 'XMLHttpRequest',
            }
        )
        if r.status_code != 200:
            return None

        resp_data = r.json()
        if resp_data['status'] != '1':
            return None

        results = []
        for item in resp_data['data']:
            results.append(dict(
                slug=item['slug'],
                name=item['title'],
                cover=item['img_url'],
                last_chapter=item['chapter_slug']
            ))

        return results
